package server.command;

import java.util.Collection;

import common.collect.Filter;
import server.network.Player;
import server.network.User;

public class UserParser extends CompletingParser {
	protected final boolean useSender;
	protected final UserPolicy policy;
	
	public UserParser(String name, boolean useSender, UserPolicy policy) {
		super(name);
		this.useSender = useSender;
		this.policy = policy;
	}

	public Object parse(CommandEnvironment env, String input) {
		User user = env.getServer().getUser(input);
		if(user == null)
			throw new RunException("Nutzer '%s' wurde nicht gefunden", input);
		if(!this.policy.applies(env, env.getExecutor(), user))
			throw new RunException(this.policy.getDenyMessage(user));
		return user;
	}
	
	public Object getDefault(CommandEnvironment env) {
		return this.useSender && env.getExecutor().isPlayer() && this.policy.applies(env, env.getExecutor(), (Player)env.getExecutor()) ? (Player)env.getExecutor() : null;
	}
	
	public Collection<String> getCompletions(CommandEnvironment env) {
		return Filter.filter(env.getServer().getAllUserNames(), user -> this.policy.applies(env, env.getExecutor(), env.getServer().getUser(user))); // add Lists.newArrayList if modifying!
	}

	public Class<?> getTypeClass(boolean required) {
		return User.class;
	}
}
