package server.command.commands;

import java.util.List;

import common.entity.Entity;
import common.tags.TagObject;
import common.util.BlockPos;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.UserPolicy;

public class CommandEntity extends Command {
	public CommandEntity() {
		super("entity");
		
		this.addEntityList("entities", false, UserPolicy.NO_PLAYERS);
		this.setParamsOptional();
		this.addTag("tag");
	}
	
	public Object exec(CommandEnvironment env, Executor exec, List<Entity> entities, TagObject tag) {
		int done = 0;
		for(Entity entity : entities) {
			if(entity.isEntityAlive()) {
				BlockPos pos = entity.getPosition();
				TagObject etag = new TagObject();
				entity.writeTags(etag);
				if(tag == null) {
					exec.log("************************************************************");
					exec.log("Daten von %s bei %d, %d, %d in %s:", entity.getCommandName(), pos.getX(), pos.getY(), pos.getZ(), entity.worldObj.dimension.getFormattedName(false));
					exec.log(etag.format(4));
				}
				else {
					etag.merge(tag);
					entity.readTags(etag);
					exec.log("Daten von %s bei %d, %d, %d in %s geändert", entity.getCommandName(), pos.getX(), pos.getY(), pos.getZ(), entity.worldObj.dimension.getFormattedName(false));
				}
				done++;
			}
		}
		if(tag != null && done > 1)
			exec.log("Daten von %d Objekten geändert", done);
		return done;
	}
}
