package server.command.commands;

import java.security.PublicKey;

import common.color.TextColor;
import common.network.IPlayer;
import common.util.EncryptUtil;
import common.util.Pair;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.RunException;
import server.network.Player;
import server.network.User;
import server.util.Form;

public class CommandRegkey extends Command {
	public CommandRegkey() {
		super("regkey");

		this.addString("username", false, null, IPlayer.MAX_USER_LENGTH, IPlayer.VALID_USER);
		this.setParamsOptional();
		this.addString("keySpec", false);
		this.addString("keyData", false);
		this.addString("keyHash", false);
		this.addString("keyName", false);
		this.addFlag("admin", 'a');
	}
	
	public void exec(CommandEnvironment env, Executor exec, String username, String keySpec, String keyData, String keyHash, String keyName, boolean admin) {
		if(env.getServer().getUser(username) != null)
			throw new RunException("Ein Spieler mit diesem Nutzernamen ist bereits registriert");
		if(env.getServer().loadPlayerData(username) != null)
			throw new RunException("Ein Spieler mit diesem Nutzernamen hat bereits Spielerdaten, ist aber nicht registriert");
		if(exec.isPlayer()) {
			if(keySpec != null || keyData != null || keyHash != null || keyName != null)
				throw new RunException("Bei Verwendung als Spieler darf kein Schlüssel angegeben werden");
			((Player)exec).displayForm(new Form() {
				private Field keyField;
				
				protected void init() {
					this.keyField = this.addField("Pubkey", 1, 960, "");
				}
				
				public String getTitle() {
					return "Spieler " + username + " registrieren";
				}
				
				protected void accept() {
					if(!((Player)exec).isAdmin() || env.getServer().getUser(username) != null || env.getServer().loadPlayerData(username) != null) {
						exec.log(TextColor.DRED + "Ein Fehler ist aufgetreten");
						return;
					}
					Pair<PublicKey, String> key;
					try {
						key = EncryptUtil.parseArmoredPubkey(this.keyField.get());
					}
					catch(IllegalArgumentException e) {
						exec.log(TextColor.RED + "Ungültiger Schlüssel");
						return;
					}
					User user = new User(username);
					user.setPubkey(key.first());
					user.setAdmin(admin);
					env.getServer().addUser(user);
					exec.log(TextColor.GREEN + "Spieler %s registriert", username);
				}
			});
		}
		else if(exec.isConsole()) {
			if(keySpec == null || keyData == null || keyHash == null)
				throw new RunException("Bei Verwendung in der Konsole muss ein Schlüssel angegeben werden");
			Pair<PublicKey, String> key;
			try {
				key = EncryptUtil.parseArmoredPubkey(keySpec + " " + keyData + " " + keyHash + (keyName == null ? null : " " + keyName));
			}
			catch(IllegalArgumentException e) {
				throw new RunException(e, "Ungültiger Schlüssel");
			}
			User user = new User(username);
			user.setPubkey(key.first());
			user.setAdmin(admin);
			env.getServer().addUser(user);
			exec.log(TextColor.GREEN + "Spieler %s registriert", username);
		}
	}
}
