package server.vars;

import java.lang.reflect.Field;

import common.util.ExtMath;
import common.util.Var;

public class SVar {
	public final ValueType type;
	public final String def;
	public final boolean noDef;
	private final Field field;
	private final float min;
	private final float max;
	
	private Runnable callback;

	public SVar(Field field, Var value) {
		this.type = field.getType() == int.class ? ValueType.INTEGER : (field.getType() == boolean.class ? ValueType.BOOLEAN : 
			(field.getType() == String.class ? ValueType.STRING : (field.getType() == float.class ? ValueType.FLOAT : null)));
		if(this.type == null)
			throw new IllegalArgumentException(value.name() + ": Unbekannter Variablen-Typ - " + field.getType());
		this.field = field;
		this.def = this.get();
		this.noDef = value.nonDefault();
		this.min = (this.type == ValueType.INTEGER || this.type == ValueType.FLOAT) 
				? value.min() : 0;
		this.max = (this.type == ValueType.INTEGER || this.type == ValueType.FLOAT) 
				? value.max() : (this.type == ValueType.BOOLEAN ? 1 : 0);
		this.set(this.def, this.noDef, false);
	}
	
	public String get() {
		try {
			return "" + this.field.get(null);
		}
		catch(IllegalArgumentException | IllegalAccessException e) {
			throw new RuntimeException(e);
		}
	}
	
	public void setCallback(Runnable callback) {
		this.callback = callback;
	}
	
	public void set(String value, boolean noClamp, boolean update) {
		try {
			switch(this.type) {
				case STRING:
					this.field.set(null, value);
					break;
				case BOOLEAN:
					this.field.setBoolean(null, Boolean.parseBoolean(value));
					break;
				case INTEGER:
					int inum = 0;
					try {
						inum = Integer.parseInt(value);
					}
					catch(NumberFormatException e) {
					}
					this.field.setInt(null, noClamp ? inum : ExtMath.clampi(inum, (int)this.min, (int)this.max));
					break;
				case FLOAT:
					float fnum = 0.0f;
					try {
						fnum = Float.parseFloat(value);
					}
					catch(NumberFormatException e) {
					}
					if(!noClamp)
						fnum = ExtMath.clampf(fnum, this.min, this.max);
					int round = (int)(fnum * 1000.0f);
					this.field.setFloat(null, (float)round / 1000.0f);
					break;
			}
		}
		catch(IllegalArgumentException | IllegalAccessException e) {
			throw new RuntimeException(e);
		}
		if(update && this.callback != null)
			this.callback.run();
	}
}